@@ -143,8 +143,10 @@ xmlCleanupInputCallbacks(void)
         xmlInputCallbackTable[i].readcallback = NULL;
         xmlInputCallbackTable[i].closecallback = NULL;
     }
+    xmlInputCallbackInitialized = 0;
 
     xmlInputCallbackNr = 0;
+    xmlInputCallbackInitialized = 0;
 }
 
 /**
@@ -167,8 +169,10 @@ xmlCleanupOutputCallbacks(void)
         xmlOutputCallbackTable[i].writecallback = NULL;
         xmlOutputCallbackTable[i].closecallback = NULL;
     }
+    xmlOutputCallbackInitialized = 0;
 
     xmlOutputCallbackNr = 0;
+    xmlOutputCallbackInitialized = 0;
 }
 
 /************************************************************************
@@ -488,7 +492,7 @@ xmlGzfileOpenW (const char *filename, int compression) {
     char mode[15];
     gzFile fd;
 
-    sprintf(mode, "wb%d", compression);
+    snprintf(mode, sizeof(mode), "wb%d", compression);
     if (!strcmp(filename, "-")) {
         fd = gzdopen(dup(1), mode);
 	return((void *) fd);
@@ -710,9 +714,9 @@ xmlCreateZMemBuff( int compression ) {
     }
 
     /*  Set the header data.  The CRC will be needed for the trailer  */
-
     buff->crc = crc32( 0L, Z_NULL, 0 );
-    hdr_lgth = sprintf( (char *)buff->zbuff, "%c%c%c%c%c%c%c%c%c%c",
+    hdr_lgth = snprintf( (char *)buff->zbuff, buff->size,
+			"%c%c%c%c%c%c%c%c%c%c",
 			GZ_MAGIC1, GZ_MAGIC2, Z_DEFLATED, 
 			0, 0, 0, 0, 0, 0, LXML_ZLIB_OS_CODE );
     buff->zctrl.next_out  = buff->zbuff + hdr_lgth;
@@ -1178,7 +1182,7 @@ xmlIOHTTPCloseWrite( void * context, const char * http_mthd ) {
 
 	    dump_name = tempnam( NULL, "lxml" );
 	    if ( dump_name != NULL ) {
-		(void)sprintf( buffer, "%s.content", dump_name );
+		(void)snprintf( buffer, sizeof(buffer), "%s.content", dump_name );
 
 		tst_file = fopen( buffer, "w" );
 		if ( tst_file != NULL ) {
@@ -1190,7 +1194,7 @@ xmlIOHTTPCloseWrite( void * context, const char * http_mthd ) {
 		    fclose( tst_file );
 		}
 
-		(void)sprintf( buffer, "%s.reply", dump_name );
+		(void)snprintf( buffer, sizeof(buffer), "%s.reply", dump_name );
 		tst_file = fopen( buffer, "w" );
 		if ( tst_file != NULL ) {
 		    xmlGenericError( xmlGenericErrorContext,
@@ -2114,7 +2118,7 @@ xmlParserInputBufferGrow(xmlParserInputBufferPtr in, int len) {
     int res = 0;
     int nbchars = 0;
     int buffree;
-    int needSize;
+    unsigned int needSize;
 
     if ((len <= MINLEN) && (len != 4)) 
         len = MINLEN;
